/*
** Module   : SOAPFactory.java
** Abstract : Factory of SOAP objects. 
**
** Copyright (c) 2013, Golden Code Development Corporation.
** ALL RIGHTS RESERVED. Use is subject to license terms.
**
**           Golden Code Development Corporation
**                      CONFIDENTIAL
**
** -#- -I- --Date-- --------------------------------Description-----------------------------------
** 001 CA  20130216 Created initial version.
** 002 CA  20130918 Added widget-pool support. The pool name must be validated before anything 
**                  else, and a null name means the default pool is used.
*/

package com.goldencode.p2j.util;

/**
 * Adds support for the {@link #createSOAPHeader CREATE SOAP-HEADER} and  
 * {@link #createSOAPHeaderEntry(handle) CREATE SOAP-HEADER-ENTRYREF} language statements. 
 */
public class SOAPFactory
{
   /**
    * Create a SOAP-HEADER object and save it in the given handle.
    * 
    * @param    h
    *           A handle variable, field or class member where the SOAP-HEADER will be saved. 
    */
   public static void createSOAPHeader(handle h)
   {
      createSOAPHeader(h, (character) null);
   }

   /**
    * Create a SOAP-HEADER object, save it in the given handle and register it in the given 
    * named widget-pool.
    * 
    * @param    h
    *           A handle variable, field or class member where the SOAP-HEADER will be saved.
    * @param    pool
    *           The name of the widget pool. <code>null</code> if the unnamed pool must be used.
    */
   public static void createSOAPHeader(handle h, String pool)
   {
      createSOAPHeader(h, (pool == null ? null : new character(pool)));
   }

   /**
    * Create a SOAP-HEADER object, save it in the given handle and register it in the given 
    * named widget-pool.
    * 
    * @param    h
    *           A handle variable, field or class member where the SOAP-HEADER will be saved.
    * @param    pool
    *           The name of the widget pool. <code>null</code> if the unnamed pool must be used.
    */
   public static void createSOAPHeader(handle h, character pool)
   {
      // validate widget pool before anything else!!!
      if (!WidgetPool.validWidgetPool(pool))
      {
         return;
      }

      WrappedResource res = new SOAPHeaderImpl();
      h.assign(res);
      
      WidgetPool.addResource(res, pool);
   }

   /**
    * Create a SOAP-HEADER-ENTRYREF object and save it in the given handle.
    * 
    * @param    h
    *           A handle variable, field or class member where the SOAP-HEADER-ENTRYREF will be 
    *           saved.
    */
   public static void createSOAPHeaderEntry(handle h)
   {
      createSOAPHeader(h, (character) null);
   }

   /**
    * Create a SOAP-HEADER-ENTRYREF object, save it in the given handle and register it in the  
    * given named widget-pool.
    * 
    * @param    h
    *           A handle variable, field or class member where the SOAP-HEADER-ENTRYREF will be 
    *           saved.
    * @param    pool
    *           The name of the widget pool. <code>null</code> if the unnamed pool must be used.
    */
   public static void createSOAPHeaderEntry(handle h, String pool)
   {
      createSOAPHeaderEntry(h, (pool == null ? null : new character(pool)));
   }


   /**
    * Create a SOAP-HEADER-ENTRYREF object, save it in the given handle and register it in the  
    * given named widget-pool.
    * 
    * @param    h
    *           A handle variable, field or class member where the SOAP-HEADER-ENTRYREF will be 
    *           saved.
    * @param    pool
    *           The name of the widget pool. <code>null</code> if the unnamed pool must be used.
    */
   public static void createSOAPHeaderEntry(handle h, character pool)
   {
      // validate widget pool before anything else!!!
      if (!WidgetPool.validWidgetPool(pool))
      {
         return;
      }

      WrappedResource res = new SOAPHeaderEntryImpl();
      h.assign(res);
      
      WidgetPool.addResource(res, pool);
   }
}
