/*
** Module   : XmlFactory
** Abstract : DOM and SAX XML instance creation.
**
** Copyright (c) 2013, Golden Code Development Corporation.
** ALL RIGHTS RESERVED. Use is subject to license terms.
**
**           Golden Code Development Corporation
**                      CONFIDENTIAL
**
** -#- -I- --Date-- --------------------------------Description-----------------------------------
** 001 EVL 20130204 Created initial version. Major rework to use w3c backend.
** 002 EVL 20130208 Added SAX XML Support.
** 003 CA  20130308 The CREATE APIs need to receive a handle parameter, to properly initiate it,
**                  as returning the new handle instance and changing the handle reference in the
**                  caller is not the right approach.
** 004 CA  20130309 Added missing APIs related to WIDGET-POOL clause.
** 005 CA  20130311 Added more missing APIs related to WIDGET-POOL clause.
** 006 CA  20130918 Added widget-pool support. The pool name must be validated before anything 
**                  else, and a null name means the default pool is used.
*/

package com.goldencode.p2j.xml;

import org.w3c.dom.*;
import com.goldencode.p2j.util.*;

/**
 * The set of methods to work with DOM XML and SAX support implementation in P2J.
 */
public class XmlFactory
{
   /**
    * Constructs a new instance of the Progress X-Document object and
    * retrieves the handle variable associated with it.
    * 
    * @param    h
    *           The handle where the new resource will be set as the referent.
    */
   public static void createXDocument(handle h)
   {
      createXDocument(h, (character) null);
   }

   /**
    * Constructs a new instance of the Progress X-Document object and
    * retrieves the handle variable associated with it.
    * 
    * @param    h
    *           The handle where the new resource will be set as the referent.
    * @param    poolName
    *           The name of the widget pool. <code>null</code> if the unnamed pool must be used.
    */
   public static void createXDocument(handle h, String poolName)
   {
      createXDocument(h, (poolName == null ? null : new character(poolName)));
   }

   /**
    * Constructs a new instance of the Progress X-Document object and
    * retrieves the handle variable associated with it.
    * 
    * @param    h
    *           The handle where the new resource will be set as the referent.
    * @param    poolName
    *           The name of the widget pool. <code>null</code> if the unnamed pool must be used.
    */
   public static void createXDocument(handle h, character poolName)
   {
      // validate widget pool before anything else!!!
      if (!WidgetPool.validWidgetPool(poolName))
      {
         return;
      }

      WrappedResource res = new XDocumentImpl();
      h.assign(res);
      
      WidgetPool.addResource(res, poolName);
   }

   /**
    * Constructs a new instance of the Progress X-Node-Reference object and
    * retrieves the handle variable associated with it.
    * 
    * @param    h
    *           The handle where the new resource will be set as the referent.
    */
   public static void createXNodeReference(handle h)
   {
      createXNodeReference(h, (character) null);
   }

   /**
    * Constructs a new instance of the Progress X-Node-Reference object and
    * retrieves the handle variable associated with it.
    * 
    * @param    h
    *           The handle where the new resource will be set as the referent.
    * @param    poolName
    *           The name of the widget pool. <code>null</code> if the unnamed pool must be used.
    */
   public static void createXNodeReference(handle h, String poolName)
   {
      createXNodeReference(h, (poolName == null ? null : new character(poolName)));
   }

   /**
    * Constructs a new instance of the Progress X-Node-Reference object and
    * retrieves the handle variable associated with it.
    * 
    * @param    h
    *           The handle where the new resource will be set as the referent.
    * @param    poolName
    *           The name of the widget pool. <code>null</code> if the unnamed pool must be used.
    */
   public static void createXNodeReference(handle h, character poolName)
   {
      // validate widget pool before anything else!!!
      if (!WidgetPool.validWidgetPool(poolName))
      {
         return;
      }

      WrappedResource res = new XNodeRefImpl();
      h.assign(res);
      
      WidgetPool.addResource(res, poolName);
   }

   /**
    * Constructs a new instance of the Progress X-Node-Reference object and
    * retrieves the handle variable associated with it.
    *
    * @param    h
    *           The handle where the new resource will be set as the referent.
    * @param    doc
    *           The X-Document object to create X-Node under.
    * @param    name
    *           The name of the node to create.
    * @param    type
    *           The type of the node to create.
    */
   public static void createXNodeReference(handle h,
                                           Document doc,
                                           character name,
                                           character type)
   {
      String nameImpl = null;
      
      if (!name.isUnknown())
      {
         nameImpl = name.toStringMessage();
      }

      Element xelement = doc.createElement(nameImpl);

      h.assign(new XNodeRefImpl(xelement)); 
   }

   /**
    * Constructs a new instance of the Progress X-Node-Reference object and
    * retrieves the handle variable associated with it. The approach is
    * slightly different. Each node will have additional paranmeter - Name
    * Space and node will be addressed by namespace and name pair instead
    * of the simple name.
    *
    * @param    h
    *           The handle where the new resource will be set as the referent.
    * @param    doc
    *           The X-Document object to create X-Node under.
    * @param    nsUri
    *           Name space associated with the current set of names.
    * @param    name
    *           The name of the node to create.
    * @param    type
    *           The type of the node to create.
    */
   public static void createXNodeReference(handle    h,
                                           Document  doc,
                                           character nsUri,
                                           character name,
                                           character type)
   {
      String nameImpl = null;
      String nsUriImpl = null;
      
      if (!nsUri.isUnknown())
      {
         nsUriImpl = nsUri.toStringMessage();
      }
      if (!name.isUnknown())
      {
         nameImpl = name.toStringMessage();
      }
      
      Element xelement = doc.createElementNS(nsUriImpl, nameImpl);

      h.assign(new XNodeRefImpl(xelement)); 
   }

   /**
    * Constructs a new instance of the Progress Sax-Attributes object and retrieves the handle
    * variable associated with it.
    * 
    * @param    h
    *           The handle where the new resource will be set as the referent.
    */
   public static void createSaxAttributes(handle h)
   {
      createSaxAttributes(h, (character) null);
   }

   /**
    * Constructs a new instance of the Progress Sax-Attributes object and retrieves the handle
    * variable associated with it.
    *
    * @param    h
    *           The handle where the new resource will be set as the referent.
    * @param    widgetPool
    *           The name of the widget pool. <code>null</code> if the unnamed pool must be used.
    */
   public static void createSaxAttributes(handle h, String widgetPool)
   {
      createSaxAttributes(h, (widgetPool == null ? null : new character(widgetPool)));
   }

   /**
    * Constructs a new instance of the Progress Sax-Attributes object and retrieves the handle
    * variable associated with it.
    *
    * @param    h
    *           The handle where the new resource will be set as the referent.
    * @param    widgetPool
    *           The name of the widget pool. <code>null</code> if the unnamed pool must be used.
    */
   public static void createSaxAttributes(handle h, character widgetPool)
   {
      // validate widget pool before anything else!!!
      if (!WidgetPool.validWidgetPool(widgetPool))
      {
         return;
      }

      // TODO: create it
      WrappedResource res = null;
      
      WidgetPool.addResource(res, widgetPool);
   }
   
   /**
    * Constructs a new instance of the Progress Sax-Reader object and retrieves the handle
    * variable associated with it.
    * 
    * @param    h
    *           The handle where the new resource will be set as the referent.
    */
   public static void createSaxReader(handle h)
   {
      createSaxReader(h, (character) null);
   }

   /**
    * Constructs a new instance of the Progress Sax-Reader object and retrieves the handle
    * variable associated with it.
    *
    * @param    h
    *           The handle where the new resource will be set as the referent.
    * @param    widgetPool
    *           The name of the widget pool. <code>null</code> if the unnamed pool must be used.
    */
   public static void createSaxReader(handle h, String widgetPool)
   {
      createSaxReader(h, (widgetPool == null ? null : new character(widgetPool)));
   }

   /**
    * Constructs a new instance of the Progress Sax-Reader object and retrieves the handle
    * variable associated with it.
    *
    * @param    h
    *           The handle where the new resource will be set as the referent.
    * @param    widgetPool
    *           The name of the widget pool. <code>null</code> if the unnamed pool must be used.
    */
   public static void createSaxReader(handle h, character widgetPool)
   {
      // validate widget pool before anything else!!!
      if (!WidgetPool.validWidgetPool(widgetPool))
      {
         return;
      }

      // TODO: create it
      WrappedResource res = null;
      
      WidgetPool.addResource(res, widgetPool);
   }
   
   /**
    * Constructs a new instance of the Progress Sax-Writer object and retrieves the handle
    * variable associated with it.
    * 
    * @param    h
    *           The handle where the new resource will be set as the referent.
    */
   public static void createSaxWriter(handle h)
   {
      createSaxWriter(h, (character) null);
   }

   /**
    * Constructs a new instance of the Progress Sax-Writer object and retrieves the handle
    * variable associated with it.
    *
    * @param    h
    *           The handle where the new resource will be set as the referent.
    * @param    widgetPool
    *           The name of the widget pool. <code>null</code> if the unnamed pool must be used.
    */
   public static void createSaxWriter(handle h, String widgetPool)
   {
      createSaxWriter(h, (widgetPool == null ? null : new character(widgetPool)));
   }

   /**
    * Constructs a new instance of the Progress Sax-Writer object and retrieves the handle
    * variable associated with it.
    *
    * @param    h
    *           The handle where the new resource will be set as the referent.
    * @param    widgetPool
    *           The name of the widget pool. <code>null</code> if the unnamed pool must be used.
    */
   public static void createSaxWriter(handle h, character widgetPool)
   {
      // validate widget pool before anything else!!!
      if (!WidgetPool.validWidgetPool(widgetPool))
      {
         return;
      }

      // TODO: create it
      WrappedResource res = null;
      
      WidgetPool.addResource(res, widgetPool);
   }
}
